/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Comprueba si una casilla es atacada por una pieza, en base a ello comprobamos si el rey
est en jaque, basado en las caractersticas de generacin de movimientos de FirstChess,
est funcin es muchsimo ms rpida que la utilizada con mailbox por TSCP.
*/

/* comprueba si el rey est en jaque, tenemos guardadas la posicin de los reyes rb (rey 
blanco) y rn ya que vamos actualizando su posicin en hacer_movimiento y deshacer */
int	EstaEnJaque(int turno) 
{
	if (turno == BLANCO)
		return atacado(rb, BLANCO);
	else
		return atacado(rn, NEGRO);
}

/* comprueba si una casilla k es atacada por una casilla de color contraria al turno que toca
retorna 1 (VERDADERO) si es atacada y un 0 (FALSO) si no lo es */
int atacado(int k, int turno)	
{
	int				h,
                    y,
                    fila,
                    columna,
                    cturno;
    
	cturno = (BLANCO + NEGRO) - turno;     //quien nos da jaque tiene el turno contrario
    fila = FILA(k), columna = COLUMNA(k);

    // Chequea los ataques del caballo
    if (columna > 0 && fila > 1 && color[k - 17] == cturno && pieza[k - 17] == CABALLO)
        return 1;
    if (columna < 7 && fila > 1 && color[k - 15] == cturno && pieza[k - 15] == CABALLO)
        return 1;
    if (columna > 1 && fila > 0 && color[k - 10] == cturno && pieza[k - 10] == CABALLO)
        return 1;
    if (columna < 6 && fila > 0 && color[k - 6] == cturno && pieza[k - 6] == CABALLO)
        return 1;
    if (columna > 1 && fila < 7 && color[k + 6] == cturno && pieza[k + 6] == CABALLO)
        return 1;
    if (columna < 6 && fila < 7 && color[k + 10] == cturno && pieza[k + 10] == CABALLO)
        return 1;
    if (columna > 0 && fila < 6 && color[k + 15] == cturno && pieza[k + 15] == CABALLO)
        return 1;
    if (columna < 7 && fila < 6 && color[k + 17] == cturno && pieza[k + 17] == CABALLO)
        return 1;
    
	// Chequea las lineas horizontales y verticales para los ataques de dama, torre y rey
    // abajo
    y = k + 8;
    if (y < 64) {
        if (color[y] == cturno && (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y += 8; y < 64; y += 8) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // izquierda
    y = k - 1;
    h = k - columna;
    if (y >= h) {
        if (color[y] == cturno && (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y--; y >= h; y--) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // derecha
    y = k + 1;
    h = k - columna + 7;
    if (y <= h) {
        if (color[y] == cturno && (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y++; y <= h; y++) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba
    y = k - 8;
    if (y >= 0) {
        if (color[y] == cturno && (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y -= 8; y >= 0; y -= 8) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    
	// Chequea las diagonales para los ataques de dama, alfil, rey y peon
    // abajo derecha
    y = k + 9;
    if (y < 64 && COLUMNA(y) != 0) {
        if (color[y] == cturno) {
            if (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == NEGRO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y += 9; y < 64 && COLUMNA(y) != 0; y += 9) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // abajo izquierda
    y = k + 7;
    if (y < 64 && COLUMNA(y) != 7) {
        if (color[y] == cturno) {
            if (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == NEGRO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y += 7; y < 64 && COLUMNA(y) != 7; y += 7) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba izquierda
    y = k - 9;
    if (y >= 0 && COLUMNA(y) != 7) {
        if (color[y] == cturno) {
            if (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == BLANCO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y -= 9; y >= 0 && COLUMNA(y) != 7; y -= 9) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba derecha
    y = k - 7;
    if (y >= 0 && COLUMNA(y) != 0) {
        if (color[y] == cturno) {
            if (pieza[y] == REY || pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == BLANCO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y -= 7; y >= 0 && COLUMNA(y) != 0; y -= 7) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    return 0;
}

/* comprueba si una casilla k es atacada por una casilla de color contraria al turno que toca
retorna 1 (VERDADERO) si es atacada y un 0 (FALSO) si no lo es, no comprueba los ataques o
defensas por parte del rey*/
int atacado_nr(int k, int turno)	
{
	int				h,
                    y,
                    fila,
                    columna,
                    cturno;
    
	cturno = (BLANCO + NEGRO) - turno;     //quien nos da jaque tiene el turno contrario
    fila = FILA(k), columna = COLUMNA(k);

    // Chequea los ataques del caballo
    if (columna > 0 && fila > 1 && color[k - 17] == cturno && pieza[k - 17] == CABALLO)
        return 1;
    if (columna < 7 && fila > 1 && color[k - 15] == cturno && pieza[k - 15] == CABALLO)
        return 1;
    if (columna > 1 && fila > 0 && color[k - 10] == cturno && pieza[k - 10] == CABALLO)
        return 1;
    if (columna < 6 && fila > 0 && color[k - 6] == cturno && pieza[k - 6] == CABALLO)
        return 1;
    if (columna > 1 && fila < 7 && color[k + 6] == cturno && pieza[k + 6] == CABALLO)
        return 1;
    if (columna < 6 && fila < 7 && color[k + 10] == cturno && pieza[k + 10] == CABALLO)
        return 1;
    if (columna > 0 && fila < 6 && color[k + 15] == cturno && pieza[k + 15] == CABALLO)
        return 1;
    if (columna < 7 && fila < 6 && color[k + 17] == cturno && pieza[k + 17] == CABALLO)
        return 1;
    
	// Chequea las lineas horizontales y verticales para los ataques de dama, torre
    // abajo
    y = k + 8;
    if (y < 64) {
        if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y += 8; y < 64; y += 8) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // izquierda
    y = k - 1;
    h = k - columna;
    if (y >= h) {
        if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y--; y >= h; y--) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // derecha
    y = k + 1;
    h = k - columna + 7;
    if (y <= h) {
        if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y++; y <= h; y++) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba
    y = k - 8;
    if (y >= 0) {
        if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
            return 1;
        if (pieza[y] == VACIO)
            for (y -= 8; y >= 0; y -= 8) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == TORRE))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    
	// Chequea las diagonales para los ataques de dama, alfil, y peon
    // abajo derecha
    y = k + 9;
    if (y < 64 && COLUMNA(y) != 0) {
        if (color[y] == cturno) {
            if (pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == NEGRO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y += 9; y < 64 && COLUMNA(y) != 0; y += 9) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // abajo izquierda
    y = k + 7;
    if (y < 64 && COLUMNA(y) != 7) {
        if (color[y] == cturno) {
            if (pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == NEGRO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y += 7; y < 64 && COLUMNA(y) != 7; y += 7) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba izquierda
    y = k - 9;
    if (y >= 0 && COLUMNA(y) != 7) {
        if (color[y] == cturno) {
            if (pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == BLANCO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y -= 9; y >= 0 && COLUMNA(y) != 7; y -= 9) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    // arriba derecha
    y = k - 7;
    if (y >= 0 && COLUMNA(y) != 0) {
        if (color[y] == cturno) {
            if (pieza[y] == DAMA || pieza[y] == ALFIL)
                return 1;
            if (turno == BLANCO && pieza[y] == PEON)
                return 1;
        }
        if (pieza[y] == VACIO)
            for (y -= 7; y >= 0 && COLUMNA(y) != 0; y -= 7) {
                if (color[y] == cturno && (pieza[y] == DAMA || pieza[y] == ALFIL))
                    return 1;
                if (pieza[y] != VACIO)
                    break;
            }
    }
    return 0;
}