/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Funciones para el manejo del libro de aperturas
*/

//definimos el archivo para el libro de aperturas
FILE *libro_aperturas;

//abrimos el libro de aperturas
void abrir_libro()
{
	//abrimos el libro para las blancas o negras segn lo que juegue el motor
	if (motor == BLANCO) {
		printf("Book: %s\n", libroblanco);
		libro_aperturas = fopen(libroblanco, "r");
	}
	else {
		printf("Book: %s\n", libronegro);
		libro_aperturas = fopen(libronegro, "r");
	}
	/* si no existen los archivos de libros definidos en el fichero de configuracin, lo
	indicamos */
	if (!libro_aperturas)
		printf("Opening book no found.\n");
}

//cerramos el libro de aperturas
void cerrar_libro()
{
	if (libro_aperturas)
		fclose(libro_aperturas);
	libro_aperturas = NULL;
}

/*la siguiente funcin la utilizamos para generar una linea con los movimientos jugados en la
partida, por ejemplo, "e2e4e7e5g1f3...", convierte un movimiento de la lista de jugadas en una 
cadena de texto, la funcin est preparada para utilizar movimientos con promocin, aunque el 
libro no los utiliza*/
char *movimiento_cadena(movimiento m)
{
	static char str[6];
	char c = 0;

	//letra a poner si hay una promocin
	if (m.tipo >= PROMOCION_A_DAMA) {
		switch (m.tipo) {
			case PROMOCION_A_DAMA:
				c = 'q';
				break;
			case PROMOCION_A_TORRE:
				c = 'r';
				break;
			case PROMOCION_A_ALFIL:
				c = 'b';
				break;
			case PROMOCION_A_CABALLO:
				c = 'n';
				break;
		}
		sprintf(str, "%c%d%c%d%c",
				COLUMNA(m.de) + 'a',
				8 - FILA(m.de),
				COLUMNA(m.a) + 'a',
				8 - FILA(m.a),
				c);
	}
	else
		sprintf(str, "%c%d%c%d",
				COLUMNA(m.de) + 'a',
				8 - FILA(m.de),
				COLUMNA(m.a) + 'a',
				8 - FILA(m.a));
	return str;
}

/*comprobamos si una cadena s1 est contenido en una cadena s2, la utilizamos para saber si
la linea con los movimientos jugados hasta el momento aparece en alguna linea del libro de
aperturas*/
BOOL es_contenida(char *s1, char *s2)
{
	int i;

	for (i = 0; i < (signed int)strlen(s1); ++i)
		if (s2[i] == '\0' || s2[i] != s1[i])
			return FALSO;
	return VERDADERO;
}

/*esta es la funcin que nos devuelve una cadena con el movimiento del libro, si no hay
movimiento en el libro devuelve una cadena con un 0*/
char *movimiento_libro()
{
	int i, j, e;
	char variante[256];			//linea con los movimientos jugados hasta el momento
	char variante_libro[256];	//cada una de las lneas del libro de aperturas	
	int posibilidades = 0;		//cuantas veces encontramos la linea jugada en el libro	
	char m_replica[4];			//aqu guardo el siguiente movimiento tras encontrarse una lnea en el libro
	char m_posibles[20000];		//cuando encuentro un movimiento lo voy guardando aqu aunque est repetido

	/*si no hay libro de aperturas o hemos superado la jugada 32 o estamos editando una
	posicin, entonces no retornamos ningn movimiento*/
	if (!libro_aperturas || njugadas > 31 || !usa_libro) {
		return "0";
	}

	// variante con los movimientos jugados hasta el momento ejemplo "e2e4e7e5g1f3..."
	variante[0] = '\0';
	j = 0;
	for (i = 0; i < njugadas; ++i)
		j += sprintf(variante + j, "%s", movimiento_cadena(jugadas[i].m));
	//printf("variante: %s\n", variante);

	/*si el motor juega blancas, tenemos entonces una linea con 0 movimientos, quizs podamos
	tener un problema para retornar uno de los movimientos del libro, as que obligamos a que
	el programa juegue uno de los siguientes (al azar)*/
	if (!njugadas) {
	  srand(time(NULL));
      e = rand() % 4;
      if (!e)
         return "e2e4";
      else if (e == 1)
         return "d2d4";
      else if (e == 2)
         return "c2c4";
      else
         return "g1f3";
	}

	//nos situamos al inicio del libro
	fseek(libro_aperturas, 0, SEEK_SET);
	m_replica[0] = '\0';
	m_posibles[0] = '\0';

	//leemos cada variante del libro
	while (fgets(variante_libro, 256, libro_aperturas)) {
		// si la linea de judadas de la partida es encontrada en el libro
		if (es_contenida(variante, variante_libro)) {
			//pasamos el siguiente movimiento y lo guardamos en m_replica
			strncpy(m_replica, &variante_libro[strlen(variante)], 4);
			//guardamos en m_posibles todos los posibles m_replica no repetidos
			strncat(m_posibles, m_replica, 4);
			++posibilidades;	//cuantas posibilidades de jugar tenemos
		}
	}

	//printf("m_posibles: %s\n", m_posibles);
	//printf("Posibilidades %d\n", posibilidades);

	// si no hay  movimientos en el libro retornamos un 0
	if (posibilidades == 0)
		return "0";

	/*elegimos uno de los movimientos almacenados en m_posibles al azar, para ello calculamos
	un nmero aleatorio multiplo de 4 ya que cada movimiento est formado por 4 caracteres*/
	srand(time(NULL));
	e = rand() % posibilidades;
	e *= 4;

	/*copiamos el movimiento elegido a m_seleccionado*/
	m_seleccionado[0] = m_posibles[e];
	m_seleccionado[1] = m_posibles[e+1];
	m_seleccionado[2] = m_posibles[e+2];
	m_seleccionado[3] = m_posibles[e+3];

	/*no se porque pero tengo que imprimir m_seleccionado o sino el programa no asigna bien*/
	printf("Book move: %s\n", m_seleccionado);

	//retornamos el movimiento seleccionado
	return m_seleccionado;	
}
