/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Constantes, variables y prototipos
*/

/*
****************************************************************************
* CONSTANTES		                                                       *
****************************************************************************
*/

//nombre y fecha del programa
const char nombre[]="DanaSah v2.73";
const char fecha[]="13/11/2006";

/*definimos las 6 piezas, sera mejor definir piezas blancas y negras por separado ya que
luego sera menos costoso hacer comparaciones, una comparacin en lugar de 2*/
#define	PEON    0
#define	CABALLO 1
#define	ALFIL	2
#define	TORRE	3
#define	DAMA	4
#define	REY		5
#define	VACIO   6	// VACIO es color de pieza y tipo de pieza

// colores de las piezas y turnos
#define	BLANCO  1	
#define	NEGRO   0

#define VERDADERO	1
#define FALSO		0

// valores de las piezas
#define	VALOR_PEON		100		
#define	VALOR_CABALLO   315
#define	VALOR_ALFIL		320
#define	VALOR_TORRE     500
#define	VALOR_DAMA		950
#define	VALOR_REY		9999	// este valor no llego a utilizarlo
#define	VALOR_VACIO		0
#define	MATE            9999

#define MAX_PROFUNDIDAD	64		// mxima ply con la que pensar
#define STACK			2048	// mximo nmero de movimientos que se generan en el arbol y que guardamos

#define COLUMNA(x)	(x & 7)		// columna
#define FILA(y)		(y >> 3)	// fila

// tipos de movimientos
#define NINGUNO					0
#define NORMAL					1
#define ENROQUE					2
#define AL_PASO					3
#define PEON_AVANZA_2			4
#define PROMOCION_A_DAMA		5
#define PROMOCION_A_TORRE		6
#define PROMOCION_A_ALFIL		7
#define PROMOCION_A_CABALLO		8

// TTables
#define UPPER 1			//un valor ms alto
#define LOWER 2			//un valor ms bajo
#define EXACT 3			//un valor exacto
#define HMISS 4			//un valor desconocido
#define avoid_null	5	//evitar movimiento nulo	

// fases del juego segn la cantidad de material
//#define APERTURA	((2*VALOR_DAMA)+(4*VALOR_TORRE)+(4*VALOR_ALFIL)+(4*VALOR_CABALLO)+(16*VALOR_PEON))	
#define MEDIOJUEGO	((2*VALOR_DAMA)+(3*VALOR_TORRE)+(2*VALOR_ALFIL)+(12*VALOR_PEON))
#define FINAL		((2*VALOR_DAMA)+VALOR_TORRE+VALOR_PEON)

#define STARTPOS		"rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"

// Para Visual C
typedef unsigned __int64 U64;
typedef unsigned __int32 U32;

// Para GCC
//typedef unsigned long long U64;
//typedef unsigned long U32;

#define MIN(x,y) ((x) < (y) ? (x) : (y))


/*
****************************************************************************
* VARIABLES											                       *
****************************************************************************
*/
int turno;			// quien mueve blanco o negro
int motor;			// color con el que juega el programa
int ponder;			// piensa en el tiempo del contrario
int hacer_ponder = FALSO;	

int max_profundidad;// nivel de busqueda en el rbol
int depthnull;		// profundidad para el movimiento nulo
int sigue_Pv;

int max_tiempo;		// mximo de tiempo disponible para un movimiento en milisegundos
int min_tiempo;		// mnimo tiempo necesario para empezar el siguiente nivel
int mps;			// nmero de jugadas a realizar para cumplir el control de tiempo
int base;			// tiempo base en minutos para cumplir el control de tiempo
int inc;			// tiempo de incremento para el reloj en segundos despues de jugar
int quedan;			// nmero de movimientos que quedan para el control de tiempo
int tiempo_inicio;	// momento en el que empezamos un movimiento
int tiempo_final;	// el tiempo final ser el inicial + tiempo por movimiento

int para_busqueda;	// para poder salir de la bsqueda si termina el tiempo
int para_ponder;	// para poder salir del ponder
int salir;			// para terminar una partida si recibimos mate

long nodos;			// nmero de nodos buscados
int ply;			// nivel de profundidad que estamos en el alphabeta
int njugadas;		// nmero de movimientos en la partida (blancos+negros)

int puntos;			// evaluacin de una posicin en centipeones
int puntosblanco;	// turno blanco, puntos = puntosblanco-puntosnegro
int puntosnegro;	// turno negro, puntos = puntosnegro-puntosblanco
int material;		// la suma de los puntos del blanco y del negro

int	piezasblanco;	// puntos que suman las piezas blancas del tablero (no peones)
int	piezasnegro;	// puntos que suman las piezas negras del tablero (no peones)
int	peonesblanco;	// puntos que suman los peones blancos del tablero
int	peonesnegro;	// puntos que suman los peones negros del tablero

int peonesblancos, peonesnegros;		// nmero de peones blancos y negros
int caballosblancos, caballosnegros;	// nmero de caballos blancos y negros
int alfilesblancos, alfilesnegros;		// nmero de alfiles blancos y negros
int torresblancas, torresnegras;		// nmero de torres blancas y negras
int damasblancas, damasnegras;			// nmero de damas blancas y negras

int rb;				// posicin del rey blanco
int rn;				// posicin del rey negro

int distancia[64][64];

/*guardamos la fila del peon para cada uno de los bandos, suponemos 10 columnas en lugar de
ocho para luego hacer ms fcil la comprobacin de peones en columnas adyacentes*/
int fila_peon[2][10];	
int aislado;			//guarda si el pen est aislado

// variable valor_pieza utilizada en el clculo de material
int	valor_pieza[7] = {VALOR_PEON, VALOR_CABALLO, VALOR_ALFIL, VALOR_TORRE, VALOR_DAMA, VALOR_REY, VALOR_VACIO};

int pieza[64], color[64]; // donde guardarmos la posicin del tablero

int enroque;	// posibilidad para el enroque

int alpaso; 	// posibilidad para comer al paso
int regla50;    // contador para la regla de los 50 movimientos

// variables para almacenar una posicin, utilizas en las tablas hash
U64 hash_piezas[2][6][64]; //color, tipo de pieza, casilla
U64 hash_turno;
U64 hash_alpaso[64];

U64 hash;

// TTables
int TTSize;		// tamao para las tablas hash

// ETables
int ETSize;		// tamao para la evaluation cache

/* Elemento de la evaluacin de cache */
typedef struct {
	U64 Hash;
	int Bound;
} EType;

char libroblanco[256], libronegro[256];	// nombres para los libros de aperturas
int usa_libro;	// determina si usuamos el libro de aperturas o no
char m_seleccionado[5];		//cadena con el movimiento que retornamos

char egbb_path[256];	// ruta donde estn situadas las bitbases

char fenstring[256];

// posicin inicial de las piezas
int inicial_pieza[64] = {
	TORRE, CABALLO,	ALFIL, DAMA, REY, ALFIL, CABALLO, TORRE,
    PEON, PEON,	PEON, PEON,	PEON, PEON,	PEON, PEON,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    PEON, PEON, PEON, PEON, PEON, PEON, PEON, PEON,
    TORRE, CABALLO, ALFIL, DAMA, REY, ALFIL, CABALLO, TORRE
	};
// colores iniciales
int inicial_color[64] = {
    NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO,
    NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO, NEGRO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO, VACIO,
    BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO,
    BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO, BLANCO
	};

/*vector utilizado para actualizar los enroques despus de mover una pieza (TSCP)
Si movemos cualquier pieza que no sea el rey o torre seguiremos teniendo intactas las posi-
bilidades de enroque (15), si movemos una torre o el rey perderemos alguno de los enroques */
int enroque_mascara[64] = {
	 7, 15, 15, 15,  3, 15, 15, 11,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	13, 15, 15, 15, 12, 15, 15, 14
};

int alfilblanco, alfilnegro;	// para saber si los dos alfiles estn en la misma diagonal

/* un movimiento tiene 4 caractersticas, casilla inicial, final, tipo de movimiento que
se trata y un valor dado durante la generacin para luego poder ordenarlo */
typedef struct estructura_movimiento {
    int  de,
		 a,
         tipo;
	int valor;
}	movimiento;

/* una jugada tiene las siguientes caractersticas, el movimiento realizado, pieza capturada
por si tenemos que deshacer el movimiento, si tiene posibilidad de enroque, al paso, lleva
cuenta de la regla de los 50 movimientos y asociamos un numero nico a la posicin */
typedef struct estrutura_jugada {
    movimiento	m;
    int	captura,
		enroque,
		alpaso,
		regla50;
	U64 hash;
}	jugada;

/* Elemento de la tabla hash, en la tabla hash guardamos el nmero nico asociado con la
posicin, el score para la posicin, tipo de valor guardado (si es ms alto, bajo o exacto),
la profundidad en la que nos encontramos y el mejor movimiento a jugar*/
typedef struct {
	U64 Hash;
	int Type;
	int Depth;
	int Bound;
	movimiento Mov;
} TType;

/* a "triangular" PV array; for a good explanation of why a triangular
   array is needed, see "How Computers Play Chess" by Levy and Newborn. */
movimiento pv[MAX_PROFUNDIDAD][MAX_PROFUNDIDAD];
int pv_longuitud[MAX_PROFUNDIDAD];

jugada jugadas[STACK];		// almacenamos las jugadas que se van produciendo

movimiento killer1[MAX_PROFUNDIDAD];		// guarda los movimientos que producen un cuttoff
movimiento killer2[MAX_PROFUNDIDAD];
movimiento matekiller[MAX_PROFUNDIDAD];	// movimiento que amenaza mate

movimiento no_move = {0,0,0,0};

int Historia[64][64];		// utilizado para la ordenacin de movimientos

/*
****************************************************************************
* PROTOTIPOS		                                                       *
****************************************************************************
*/

// main
int	main();

// book
void abrir_libro();
void cerrar_libro();
char *movimiento_cadena(movimiento m);
BOOL es_contenida(char *s1, char *s2);
char *movimiento_libro();

// hash
void inicia_hash();
void poner_hash();
void clear_hash(void);
void store_hash(int score, int alpha, int beta, int depth, movimiento mov);
int probe_hash(int *type, int alpha, int beta, int depth, movimiento *mov);
void store_eval(int score);
int probe_eval();
void alloc_hash(void);
void free_hash(void);

// init
void leer_inifile();
void IniciaTablero();
int get_ms();

// attack
int	EstaEnJaque(int turno);
int atacado(int k, int turno);
int atacado_nr(int k, int turno);

// genmoves
void Generar_Movimiento_PVS(int de, int a, int tipo, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Normal_PVS(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_PEON_PVS(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_PEON2_PVS(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Enroque_PVS(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Alpaso_PVS(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Capturas(int de, int a, int tipo, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Normal_Capturas(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_PEON_Capturas(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_PEON2_Capturas(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Enroque_Capturas(int de, int a, movimiento *plista, int *pcuenta);
void Generar_Movimiento_Alpaso_Capturas(int de, int a, movimiento *plista, int *pcuenta);
int	Generar(int turno, movimiento *plista);
int	Generar_Capturas(int turno, movimiento *plista);
int Hacer_Movimiento(movimiento m);
void Deshacer();

// eval
int Evaluar(int alpha, int beta);
void Calcula_material();
void Evalua_material();
void Busca_peones();
int bpf(int f);
int rey_blanco(int sq);
int npf(int f);
int rey_negro(int sq);
int peones_blancos(int i);
int peones_negros(int i);
void Evalua_posicion_final();
void Evalua_posicion_mediojuego();
void Evalua_peones_bloqueados();
void Evalua_alfiles_atrapados();
void Evalua_caballos_atrapados();
void Cambios_mediojuego();
void Cambios_final();
void Evalua_seguridad_rey();
int Movilidad_Caballo(int pos);
int Movilidad_Alfil(int pos);
int Movilidad_Torre(int pos);
int Movilidad_Dama(int pos);
int tablas();
int repeticion();

// search
movimiento MotorPiensa(int max_profundidad, int output);
int	PVS(int alpha, int beta, int depth, int hacer_nulo);
int	PVS_no_completo(int alpha, int beta, int depth);
int Extiende_capturas(int alpha, int beta);
void Ordena_pv(int cuenta, movimiento *lista);
void Actualiza_pv(movimiento pvmov);
void OrdenaMovimiento(int de, int cuenta, movimiento *lista, movimiento hmove);
void OrdenaMovimiento2(int de, int cuenta, movimiento *lista);

//ponder
movimiento pMotorPiensa(int max_profundidad, int output);
int	pPVS(int alpha, int beta, int depth, int hacer_nulo);
int	pPVS_no_completo(int alpha, int beta, int depth);
int pExtiende_capturas(int alpha, int beta);

// gui_eng
int Comprobar_Tipo_Movimiento(int de, int a, char *s);
void Comprobar_Movimiento(char *s);
int bioskey();

// eng-gui
void print_pv(int depth, int score);
void Enviar_Movimiento(movimiento m);
void imprime_resultado(int cuenta, movimiento *lista);

// xboard
void xboard();

// Random numbers routines
U32 Rand32 (void);
U64 Rand64 (void);

// fen
void SetBoard(char *string);

/*
BitBases
*/
#define USE_EGBB _WIN32

#if defined USE_EGBB

#define _WKING     1
#define _WQUEEN    2
#define _WROOK     3
#define _WBISHOP   4
#define _WKNIGHT   5
#define _WPAWN     6
#define _BKING     7
#define _BQUEEN    8
#define _BROOK     9
#define _BBISHOP  10
#define _BKNIGHT  11
#define _BPAWN    12
#define _NOTFOUND 99999

typedef int (*PPROBE_EGBB) (int player, int w_king, int b_king,
			   int piece1, int square1,int piece2,int square2);
extern PPROBE_EGBB probe_egbb;
extern int egbb_is_loaded;
extern void LoadEgbbLibrary(char* path);

#endif
/*
end BitBases
*/