/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Funciones para generar y realizar un movimiento, estn basadas en el generador de
movimientos del programa FirstChess, se ha aadido un generador de capturas para
evitar el efecto horizonte y la generacin de enroques y al paso.
*/

//Generamos movimientos para la PVS
void Generar_Movimiento_PVS(int de, int a, int tipo, movimiento *plista, int *pcuenta)
{
	movimiento	m;
	
	/*de un movimiento guardamos la casilla inicial, final, tipo de movimiento y valor
	de lo bueno que podra ser el movimiento para luego poder ordenarlo*/
	m.de = de;	
	m.a = a;
	m.tipo = tipo;
	m.valor = 0;
	
	/*hacemos que puntuen ms el matekiller, las capturas de ganancia, promociones a dama con
	y sin captura, capturas buenas y malas, movimientos killer y dems movimientos por
	historia*/
	if (m.tipo >= PROMOCION_A_DAMA) { //si hay promocion
		if (m.tipo == PROMOCION_A_DAMA) {
			if (color[a] != VACIO) 
				m.valor = 1000075;	//promocionamos a dama con captura
			else 
				m.valor = 1000050;	//promocionamos a dama sin captura
		}
		else if (m.tipo == PROMOCION_A_CABALLO) m.valor = 40000;
		else if (m.tipo == PROMOCION_A_TORRE) m.valor = 30000;
		else if (m.tipo == PROMOCION_A_ALFIL) m.valor = 20000;
	}
	else { //no hay promocin
		if ((color[a] != VACIO))  //hay captura, ordenamos por MVV / LVA
				m.valor = 1000000 + (pieza[a]*100) - (pieza[de]);
		else {
			m.valor += Historia[de][a]; //history heuristic
			// los movimientos killer
			if ((m.de == matekiller[ply].de) && (m.a == matekiller[ply].a))
				m.valor = 1500000;
			if ((m.de == killer1[ply].de) && (m.a == killer1[ply].a))
				m.valor += 300000;
			if ((m.de == killer1[ply-2].de) && (m.a == killer1[ply-2].a))
				m.valor += 200000;
			if ((m.de == killer2[ply].de) && (m.a == killer2[ply].a))
				m.valor += 100000;
			if ((m.de == killer2[ply-2].de) && (m.a == killer2[ply-2].a))
				m.valor += 50000;
		}
	}

	plista[*pcuenta] = m;
	*pcuenta = *pcuenta + 1;
}

void Generar_Movimiento_Normal_PVS(int de, int a, movimiento *plista, int *pcuenta)
{
    Generar_Movimiento_PVS(de, a, NORMAL, plista, pcuenta);
}

void Generar_Movimiento_PEON_PVS(int de, int a, movimiento *plista, int *pcuenta)
{
    if (a > 7 && a < 56) {
		Generar_Movimiento_PVS(de, a, NORMAL, plista, pcuenta);
	} else {
		Generar_Movimiento_PVS(de, a, PROMOCION_A_DAMA, plista, pcuenta);
        Generar_Movimiento_PVS(de, a, PROMOCION_A_TORRE, plista, pcuenta);
        Generar_Movimiento_PVS(de, a, PROMOCION_A_ALFIL, plista, pcuenta);
        Generar_Movimiento_PVS(de, a, PROMOCION_A_CABALLO, plista, pcuenta);
	}
}

void Generar_Movimiento_PEON2_PVS(int de, int a, movimiento *plista, int *pcuenta)
{
	Generar_Movimiento_PVS(de, a, PEON_AVANZA_2, plista, pcuenta);
}

void Generar_Movimiento_Enroque_PVS(int de, int a, movimiento *plista, int *pcuenta)
{
	Generar_Movimiento_PVS(de, a, ENROQUE, plista, pcuenta);
}

void Generar_Movimiento_Alpaso_PVS(int de, int a, movimiento *plista, int *pcuenta)
{
	Generar_Movimiento_PVS(de, a, AL_PASO, plista, pcuenta);
}

//ahora generamos movimientos pero solo para las capturas y promocin a dama
void Generar_Movimiento_Capturas(int de, int a, int tipo, movimiento *plista, int *pcuenta)
{
	movimiento	m;
	
	m.de = de;	//de un movimiento guardamos la casilla inicial, final, tipo de movimiento y valor	
	m.a = a;
	m.tipo = tipo;
	m.valor = 0;
	if (m.tipo >= PROMOCION_A_DAMA) { //si hay promocion
		if (m.tipo == PROMOCION_A_DAMA) {
			if (color[a] != VACIO) 
				m.valor = 1000075;
			else 
				m.valor = 1000050;
		}
	}
	else { //no hay promocin
		if ((color[a] != VACIO))  //hay captura, ordenamos por MVV / LVA
			m.valor = 1000000 + (pieza[a]*100) - (pieza[de]);
	}
	plista[*pcuenta] = m;
	*pcuenta = *pcuenta + 1;
}

void    Generar_Movimiento_Normal_Capturas(int de, int a, movimiento *plista, int *pcuenta)
{
    Generar_Movimiento_Capturas(de, a, NORMAL, plista, pcuenta);
}

void    Generar_Movimiento_PEON_Capturas(int de, int a, movimiento *plista, int *pcuenta)
{
    if (a > 7 && a < 56) {
		Generar_Movimiento_Capturas(de, a, NORMAL, plista, pcuenta);
	} else {
		Generar_Movimiento_Capturas(de, a, PROMOCION_A_DAMA, plista, pcuenta);
	}
}

void    Generar_Movimiento_Alpaso_Capturas(int de, int a, movimiento *plista, int *pcuenta)
{
	Generar_Movimiento_Capturas(de, a, AL_PASO, plista, pcuenta);
}

/* Generar todos los pseudo-movimientos y retornar el nmero de ellos.
 No comprueba si nos quedamos en jaque */
int	Generar(int turno, movimiento *plista)
{
    int i, k, y,
        fila, columna,
		cuenta;
    
	cuenta = 0;
    for (i = 0; i < 64; i++) {
        if (color[i] == turno) {
            switch (pieza[i]) {
            case PEON:
                columna = COLUMNA(i);
                fila = FILA(i);
                if (turno == NEGRO) {
                    if (columna && color[i + 7] == BLANCO)
                        Generar_Movimiento_PEON_PVS(i, i + 7, plista, &cuenta); // come en diagonal - izquierda
                    if (columna < 7 && color[i + 9] == BLANCO)
                        Generar_Movimiento_PEON_PVS(i, i + 9, plista, &cuenta);	// come en diagonal - derecha 	
					if (color[i + 8] == VACIO) {  // avanza una casilla
                        Generar_Movimiento_PEON_PVS(i, i + 8, plista, &cuenta);
						if (fila == 1 && color[i + 16] == VACIO) //avanza dos casillas
							Generar_Movimiento_PEON2_PVS(i, i + 16, plista, &cuenta);
					}
                } else {
                    if (columna && color[i - 9] == NEGRO)
                        Generar_Movimiento_PEON_PVS(i, i - 9, plista, &cuenta);
                    if (columna < 7 && color[i - 7] == NEGRO)
                        Generar_Movimiento_PEON_PVS(i, i - 7, plista, &cuenta);
					if (color[i - 8] == VACIO) {
                        Generar_Movimiento_PEON_PVS(i, i - 8, plista, &cuenta);
						if (fila == 6 && color[i - 16] == VACIO)
							Generar_Movimiento_PEON2_PVS(i, i - 16, plista, &cuenta);
					}
                }
                break;

            case CABALLO:
                columna = COLUMNA(i);
                y = i - 6;
                if (y >= 0 && columna < 6 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i - 10;
                if (y >= 0 && columna > 1 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i - 15;
                if (y >= 0 && columna < 7 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i - 17;
                if (y >= 0 && columna > 0 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i + 6;
                if (y < 64 && columna > 1 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i + 10;
                if (y < 64 && columna < 6 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i + 15;
                if (y < 64 && columna > 0 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                y = i + 17;
                if (y < 64 && columna < 7 && color[y] != turno)
                    Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                break;

            case DAMA:         // La torre se mueve como ALFIL+TORRE
            case ALFIL:
                for (y = i - 9; y >= 0 && COLUMNA(y) != 7; y -= 9) {        // abajo izquierda 
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (y = i - 7; y >= 0 && COLUMNA(y) != 0; y -= 7) {        // abajo derecha
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (y = i + 9; y < 64 && COLUMNA(y) != 0; y += 9) {        // arriba derecha
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (y = i + 7; y < 64 && COLUMNA(y) != 7; y += 7) {        // arriba izquierda 
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                if (pieza[i] == ALFIL)
                    break;

            case TORRE:
                columna = COLUMNA(i);
                for (k = i - columna, y = i - 1; y >= k; y--) {					// izquierda
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (k = i - columna + 7, y = i + 1; y <= k; y++) {				// derecha
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (y = i - 8; y >= 0; y -= 8) {								// arriba 
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                for (y = i + 8; y < 64; y += 8) {								// abajo
                    if (color[y] != turno)
                        Generar_Movimiento_Normal_PVS(i, y, plista, &cuenta);
                    if (color[y] != VACIO)
                        break;
                }
                break;

            case REY:
                columna = COLUMNA(i);
                if (columna && color[i - 1] != turno)
                    Generar_Movimiento_Normal_PVS(i, i - 1, plista, &cuenta); // izquierda
                if (columna < 7 && color[i + 1] != turno)
                    Generar_Movimiento_Normal_PVS(i, i + 1, plista, &cuenta); // derecha
                if (i > 7 && color[i - 8] != turno)
                    Generar_Movimiento_Normal_PVS(i, i - 8, plista, &cuenta); // arriba 
                if (i < 56 && color[i + 8] != turno)
                    Generar_Movimiento_Normal_PVS(i, i + 8, plista, &cuenta); // abajo 
                if (columna && i > 7 && color[i - 9] != turno)
                    Generar_Movimiento_Normal_PVS(i, i - 9, plista, &cuenta); // izquierda arriba
                if (columna < 7 && i > 7 && color[i - 7] != turno)
                    Generar_Movimiento_Normal_PVS(i, i - 7, plista, &cuenta); // derecha arriba
                if (columna && i < 56 && color[i + 7] != turno)
                    Generar_Movimiento_Normal_PVS(i, i + 7, plista, &cuenta); // izquierda abajo 
                if (columna < 7 && i < 56 && color[i + 9] != turno)
                    Generar_Movimiento_Normal_PVS(i, i + 9, plista, &cuenta); // derecha abajo 
                break;
            }
        }
	}

	//generamos los enroques
	if (turno == BLANCO) {
		if (enroque & 1) 
			Generar_Movimiento_Enroque_PVS(60, 62, plista, &cuenta);
		if (enroque & 2)
			Generar_Movimiento_Enroque_PVS(60, 58, plista, &cuenta);
	}
	if (turno == NEGRO) {
		if (enroque & 4)
			Generar_Movimiento_Enroque_PVS(4, 6, plista, &cuenta);
		if (enroque & 8)
			Generar_Movimiento_Enroque_PVS(4, 2, plista, &cuenta);
	}

	// generamos al paso
	if (alpaso != -1) {
		if (turno == BLANCO) {
			if ((COLUMNA(alpaso) != 0) && (color[alpaso + 7] == BLANCO) && (pieza[alpaso + 7] == PEON))
				Generar_Movimiento_Alpaso_PVS(alpaso + 7, alpaso, plista, &cuenta);
			if ((COLUMNA(alpaso) != 7) && (color[alpaso + 9] == BLANCO) && (pieza[alpaso + 9] == PEON))
				Generar_Movimiento_Alpaso_PVS(alpaso + 9, alpaso, plista, &cuenta);
		}
		else {
			if ((COLUMNA(alpaso) != 0) && (color[alpaso - 9] == NEGRO) && (pieza[alpaso - 9] == PEON))
				Generar_Movimiento_Alpaso_PVS(alpaso - 9, alpaso, plista, &cuenta);
			if ((COLUMNA(alpaso) != 7) && (color[alpaso - 7] == NEGRO) && (pieza[alpaso - 7] == PEON))
				Generar_Movimiento_Alpaso_PVS(alpaso - 7, alpaso, plista, &cuenta);
		}
	}
    return cuenta;
}

// Generar solo movimientos de captura y promociones
// Es bsicamente una copia de Generar
int	Generar_Capturas(int turno, movimiento *plista)
{
    int i, k, y,
        fila, columna,
		cuenta, cturno;
    
	cturno = (BLANCO+NEGRO)-turno;
	cuenta = 0;
    for (i = 0; i < 64; i++) {
        if (color[i] == turno) {
            switch (pieza[i]) {
            case PEON:
                columna = COLUMNA(i);
                fila = FILA(i);
                if (turno == NEGRO) {
                    if ((color[i + 8] == VACIO) && (i>=48)) // promocin
                        Generar_Movimiento_PEON_Capturas(i, i + 8, plista, &cuenta); 
                    if (columna && color[i + 7] == BLANCO)
                        Generar_Movimiento_PEON_Capturas(i, i + 7, plista, &cuenta); 
                    if (columna < 7 && color[i + 9] == BLANCO)
                        Generar_Movimiento_PEON_Capturas(i, i + 9, plista, &cuenta);	
                } else
				if (turno == BLANCO) {
                    if ((color[i - 8] == VACIO) && (i<=15))
                        Generar_Movimiento_PEON_Capturas(i, i - 8, plista, &cuenta);
                    if (columna && color[i - 9] == NEGRO)
                        Generar_Movimiento_PEON_Capturas(i, i - 9, plista, &cuenta);
                    if (columna < 7 && color[i - 7] == NEGRO)
                        Generar_Movimiento_PEON_Capturas(i, i - 7, plista, &cuenta);
                }
                break;

            case DAMA:         
            case ALFIL:
                for (y = i - 9; y >= 0 && COLUMNA(y) != 7; y -= 9) {
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                for (y = i - 7; y >= 0 && COLUMNA(y) != 0; y -= 7) {      
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                for (y = i + 9; y < 64 && COLUMNA(y) != 0; y += 9) {      
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                for (y = i + 7; y < 64 && COLUMNA(y) != 7; y += 7) {      
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                if (pieza[i] == ALFIL)
                    break;

            case TORRE:
                columna = COLUMNA(i);
                for (k = i - columna, y = i - 1; y >= k; y--) {				
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                for (k = i - columna + 7, y = i + 1; y <= k; y++) {				
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                for (y = i - 8; y >= 0; y -= 8) {							
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
						}
                }
                for (y = i + 8; y < 64; y += 8) {							
					if (color[y] != VACIO) {
						if (color[y] != turno)
							Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                        break;
					}
                }
                break;

            case CABALLO:
                columna = COLUMNA(i);
                y = i - 6;
                if (y >= 0 && columna < 6 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i - 10;
                if (y >= 0 && columna > 1 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i - 15;
                if (y >= 0 && columna < 7 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i - 17;
                if (y >= 0 && columna > 0 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i + 6;
                if (y < 64 && columna > 1 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i + 10;
                if (y < 64 && columna < 6 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i + 15;
                if (y < 64 && columna > 0 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                y = i + 17;
                if (y < 64 && columna < 7 && (color[y] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, y, plista, &cuenta);
                break;

            case REY:
                columna = COLUMNA(i);
                if ((columna > 1) && (color[i - 1] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i - 1, plista, &cuenta);
                if ((columna < 7) && (color[i + 1] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i + 1, plista, &cuenta); 
                if ((i > 7) && (color[i - 8] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i - 8, plista, &cuenta); 
                if ((i < 56) && (color[i + 8] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i + 8, plista, &cuenta); 
                if (columna > 1 && i > 7 && (color[i - 9] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i - 9, plista, &cuenta); 
                if (columna < 7 && i > 7 && (color[i - 7] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i - 7, plista, &cuenta); 
                if (columna > 1 && i < 56 && (color[i + 7] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i + 7, plista, &cuenta); 
                if (columna < 7 && i < 56 && (color[i + 9] == cturno))
                    Generar_Movimiento_Normal_Capturas(i, i + 9, plista, &cuenta); 
                break;
            }
        }
	}

	// generamos al paso
	if (alpaso != -1) {
		if (turno == BLANCO) {
			if ((COLUMNA(alpaso) != 0) && (color[alpaso + 7] == BLANCO) && (pieza[alpaso + 7] == PEON))
				Generar_Movimiento_Alpaso_Capturas(alpaso + 7, alpaso, plista, &cuenta);
			if ((COLUMNA(alpaso) != 7) && (color[alpaso + 9] == BLANCO) && (pieza[alpaso + 9] == PEON))
				Generar_Movimiento_Alpaso_Capturas(alpaso + 9, alpaso, plista, &cuenta);
		}
		else {
			if ((COLUMNA(alpaso) != 0) && (color[alpaso - 9] == NEGRO) && (pieza[alpaso - 9] == PEON))
				Generar_Movimiento_Alpaso_Capturas(alpaso - 9, alpaso, plista, &cuenta);
			if ((COLUMNA(alpaso) != 7) && (color[alpaso - 7] == NEGRO) && (pieza[alpaso - 7] == PEON))
				Generar_Movimiento_Alpaso_Capturas(alpaso - 7, alpaso, plista, &cuenta);
		}
	}
    return cuenta;
}

int Hacer_Movimiento(movimiento m)
{
	if (m.tipo == ENROQUE) { //en caso de enroque movemos la torre
		int de, a;
		switch (m.a) {
			case 62:
				if ((color[61] != VACIO) || (color[62] != VACIO) || 
				   (atacado (60, BLANCO)) || (atacado (61, BLANCO)))
					return FALSO;
				de = 63;
				a = 61;
				break;
			case 58:
				if ((color[57] != VACIO) || (color[58] != VACIO) || (color[59] != VACIO) ||
				   (atacado (60, BLANCO)) || (atacado (59, BLANCO)))
					return FALSO;
				de = 56;
				a = 59;
				break;
			case 6:
				if ((color[5] != VACIO) || (color[6] != VACIO) || 
				   (atacado (4, NEGRO)) || (atacado (5, NEGRO)))
					return FALSO;
				de = 7;
				a = 5;
				break;
			case 2:
				if ((color[1] != VACIO) || (color[2] != VACIO) || (color[3] != VACIO) ||
				   (atacado (4, NEGRO)) || (atacado (3, NEGRO)))
					return FALSO;
				de = 0;
				a = 3;
				break;
			default: 	//no debera llegar aqu
				de = -1;
				a = -1;
				break;
		}
		color[a] = color[de];	
		pieza[a] = pieza[de];
		color[de] = VACIO;
		pieza[de] = VACIO;
	}
	
	//guardamos el movimiento, pieza capturada, enroques y al paso
    jugadas[njugadas].m = m;
    jugadas[njugadas].captura = pieza[m.a];
	jugadas[njugadas].enroque = enroque;
	jugadas[njugadas].alpaso = alpaso;
	jugadas[njugadas].regla50 = regla50;
	jugadas[njugadas].hash = hash;
    pieza[m.a] = pieza[m.de];
    pieza[m.de] = VACIO;
    color[m.a] = color[m.de];
    color[m.de] = VACIO;
	
	//actualizar enroques despues de mover un rey o torre
	enroque &= enroque_mascara[m.de] & enroque_mascara[m.a];

	//actualizar la posibilidad alpaso despus de avanzar un peon 2 casillas
	if (m.tipo == PEON_AVANZA_2) {
		if (turno == BLANCO) {
			//if ((COLUMNA(m.a - 1) >= 0 && pieza[m.a - 1] == PEON && color[m.a - 1] == NEGRO)
			//	|| (COLUMNA(m.a - 1) <= 7 && pieza[m.a + 1] == PEON && color[m.a + 1] == NEGRO))
				alpaso = m.a + 8;
		}
		else {
			//if ((COLUMNA(m.a - 1) >= 0 && pieza[m.a - 1] == PEON && color[m.a - 1] == BLANCO)
			//	|| (COLUMNA(m.a - 1) <= 7 && pieza[m.a + 1] == PEON && color[m.a + 1] == BLANCO))
				alpaso = m.a - 8;
		}
	} 
	else 
		alpaso = -1;
	
	//actualizar la regla de los 50 movimientos si no ha habido captura o movimiento de peon
	if ((jugadas[njugadas].captura != VACIO) || (pieza[m.a] == PEON))
		regla50 = 0;
	else
		regla50++;

	if (turno == NEGRO) {
		if (pieza[m.a] == REY)
			rn = m.a;
		switch(jugadas[njugadas].captura) {
			case PEON:
				peonesblancos--;
				break;
			case CABALLO:
				caballosblancos--;
				break;
			case ALFIL:
				alfilesblancos--;
				break;
			case TORRE:
				torresblancas--;
				break;
			case DAMA:
				damasblancas--;
				break;
		}
	}
	else if (turno == BLANCO) {
		if (pieza[m.a] == REY)
			rb = m.a;
		switch(jugadas[njugadas].captura) {
			case PEON:
				peonesnegros--;
				break;
			case CABALLO:
				caballosnegros--;
				break;
			case ALFIL:
				alfilesnegros--;
				break;
			case TORRE:
				torresnegras--;
				break;
			case DAMA:
				damasnegras--;
				break;
		}
	}

    //si coronamos cambia el peon por pieza
	if (m.tipo >= PROMOCION_A_DAMA) {
        switch (m.tipo) {
			case PROMOCION_A_DAMA:
				pieza[m.a] = DAMA;
				if (turno == BLANCO) {
					peonesblancos--;
					damasblancas++;
				}
				else {
					peonesnegros--;
					damasnegras++;
				}
				break;

			case PROMOCION_A_TORRE:
				pieza[m.a] = TORRE;
				if (turno == BLANCO) {
					peonesblancos--;
					torresblancas++;
				}
				else {
					peonesnegros--;
					torresnegras++;
				}
				break;

			case PROMOCION_A_ALFIL:
				pieza[m.a] = ALFIL;
				if (turno == BLANCO) {
					peonesblancos--;
					alfilesblancos++;
				}
				else {
					peonesnegros--;
					alfilesnegros++;
				}
				break;

			case PROMOCION_A_CABALLO:
				pieza[m.a] = CABALLO;
				if (turno == BLANCO) {
					peonesblancos--;
					caballosblancos++;
				}
				else {
					peonesnegros--;
					caballosnegros++;
				}
				break;
        }
    }

	// borra el peon si estamos comiendo alpaso
	if (m.tipo == AL_PASO) {
		if (turno == BLANCO) {
			color[m.a + 8] = VACIO;
			pieza[m.a + 8] = VACIO;
			peonesnegros--;
		}
		else {
			color[m.a - 8] = VACIO;
			pieza[m.a - 8] = VACIO;
			peonesblancos--;
		}
	}

	ply++;
    njugadas++; //nos preparamos para guardar la siguiente jugada
    turno = (BLANCO + NEGRO) - turno;    //da el turno al oponente
	if (EstaEnJaque(BLANCO + NEGRO - turno)) {
		Deshacer();
		return FALSO;
	}
	Calcula_material();
	poner_hash();
	return VERDADERO;
}

void Deshacer() // lo contrario a hacer un movimiento
{
	movimiento	m;
    
	turno = (BLANCO + NEGRO) - turno;
    njugadas--;
	ply--;
    m = jugadas[njugadas].m;
	enroque = jugadas[njugadas].enroque;
	alpaso = jugadas[njugadas].alpaso;
	regla50 = jugadas[njugadas].regla50;
	hash = jugadas[njugadas].hash;
    pieza[m.de] = pieza[m.a];
    pieza[m.a] = jugadas[njugadas].captura;
    color[m.de] = turno;

	if (jugadas[njugadas].captura != VACIO)
        color[m.a] = (BLANCO + NEGRO) - turno;
    else
        color[m.a] = VACIO;

	if (pieza[m.de] == REY && color[m.de] == BLANCO)
		rb = m.de;
	else if (pieza[m.de] == REY && color[m.de] == NEGRO)
		rn = m.de;


	if (color[m.a] == BLANCO) {
		switch(pieza[m.a]) {
			case PEON:
				peonesblancos++;
				break;
			case CABALLO:
				caballosblancos++;
				break;
			case ALFIL:
				alfilesblancos++;
				break;
			case TORRE:
				torresblancas++;
				break;
			case DAMA:
				damasblancas++;
				break;
		}
	}
	else if (color[m.a] == NEGRO) {
		switch(pieza[m.a]) {
			case PEON:
				peonesnegros++;
				break;
			case CABALLO:
				caballosnegros++;
				break;
			case ALFIL:
				alfilesnegros++;
				break;
			case TORRE:
				torresnegras++;
				break;
			case DAMA:
				damasnegras++;
				break;
		}
	}

	if (m.tipo >= PROMOCION_A_DAMA) { //si deshacemos una promocin colocamos el peon en
        pieza[m.de] = PEON;	//su sitio
        switch (m.tipo) {
			case PROMOCION_A_DAMA:
				if (turno == BLANCO) {
					peonesblancos++;
					damasblancas--;
				}
				else {
					peonesnegros++;
					damasnegras--;
				}
				break;

			case PROMOCION_A_TORRE:
				if (turno == BLANCO) {
					peonesblancos++;
					torresblancas--;
				}
				else {
					peonesnegros++;
					torresnegras--;
				}
				break;

			case PROMOCION_A_ALFIL:
				if (turno == BLANCO) {
					peonesblancos++;
					alfilesblancos--;
				}
				else {
					peonesnegros++;
					alfilesnegros--;
				}
				break;

			case PROMOCION_A_CABALLO:
				if (turno == BLANCO) {
					peonesblancos++;
					caballosblancos--;
				}
				else {
					peonesnegros++;
					caballosnegros--;
				}
				break;
        }
	}

	if (m.tipo == ENROQUE) { // colocamos la torre en su sitio si deshacemos el enroque
		int de, a;
		switch(m.a) {
			case 62: 
				de = 61;
				a = 63;
				break;
			case 58: 
				de = 59;
				a = 56;
				break;
			case 6: 
				de = 5;
				a = 7;
				break;
			case 2: 
				de = 3;
				a = 0;
				break;
			default:  /* no debera llegar aqu */
				de = -1;
				a = -1;
				break;
		}
		color[a] = color[de];
		pieza[a] = pieza[de];
		color[de] = VACIO;
		pieza[de] = VACIO;
	}
	if (m.tipo == AL_PASO) { //coloca el peon contrario en su sitio si deshacemos alpaso
		if (turno == BLANCO) {
			color[m.a + 8] = NEGRO;
			pieza[m.a + 8] = PEON;
			peonesnegros++;
		}
		else {
			color[m.a - 8] = BLANCO;
			pieza[m.a - 8] = PEON;
			peonesblancos++;
		}
	}
	Calcula_material();
}
