/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Funciones para manejar las tablas hash()
*/

long TTProbes;
long TTHits;
long TTStores;

long ETProbes;
long ETHits;
long ETStores;


TType *TTable;
EType *ETable;

// iniciamos las tablas con nmero aleatorios
void inicia_hash()
{
	int i, j, k;

	for (i = 0; i < 2; ++i)
		for (j = 0; j < 6; ++j)
			for (k = 0; k < 64; ++k)
				hash_piezas[i][j][k] = Rand64();
	hash_turno = Rand64();
	for (i = 0; i < 64; ++i)
		hash_alpaso[i] = Rand64();
}

// poner_hash() usa llaves de Zobrist para generar nmeros nicos
void poner_hash()
{
	int i;

	hash = 0;	
	for (i = 0; i < 64; ++i)
		if (color[i] != VACIO)
			hash ^= hash_piezas[color[i]][pieza[i]][i];
	if (turno == NEGRO)
		hash ^= hash_turno;
	if (alpaso != -1)
		hash ^= hash_alpaso[alpaso];
}

void clear_hash(void) {
	memset(TTable, 0, sizeof(TType) * TTSize);
	memset(ETable, 0, sizeof(EType) * ETSize);
};

void store_hash(int score, int alpha, int beta, int depth, movimiento hmov) {
	U64 index;
	int type;

	index = hash % TTSize;
	if (TTable[index].Depth > depth)
		return;

	if (score <= alpha) {
		type = UPPER;
		if (score < -MATE + 256)
			score = -MATE + 256;
	} 
	else if (score >= beta) {
		type = LOWER;
		if (score > MATE - 256)
			score = MATE - 256;
	} 
	else {
		type = EXACT;
		//	Ajustar puntuaciones de mate
		if (abs (score) > MATE - 256)
			if (score > 0)
				score += ply;
			else
				score -= ply;
	}

	TTStores++;

	//reemplazar el contenido de las tablas hash
	TTable[index].Hash = hash;
	TTable[index].Type = type;
	TTable[index].Depth = depth;
	TTable[index].Bound = score;
	TTable[index].Mov = hmov;
}

int probe_hash(int *type, int alpha, int beta, int depth, movimiento *mov) {
	U64 index;
	int score;

	*type = HMISS;
	*mov = no_move;

	TTProbes++;

	index = hash % TTSize;

	if (TTable[index].Hash == hash) {

		TTHits++;

		*mov = TTable[index].Mov;
		score = TTable[index].Bound;

		// Ajustar puntuaciones de mate
		if (abs (score) > MATE - 256)
			if (score > 0)
				score -= ply;
			else
				score += ply;

		// vemos si deseamos evitar movimiento nulos
		if (TTable[index].Depth >= depthnull+1 && score < beta && TTable[index].Type == UPPER)
			*type = avoid_null;

		if (TTable[index].Depth >= depth) {
			*type = TTable[index].Type;
			return score;
		}
	}

	// No se encontr
	return 0;
}

void store_eval(int score)
{
	U64 index;

	ETStores++;

	index = hash % ETSize;

	ETable[index].Hash = hash;
	ETable[index].Bound = score;
	
	return;
}

int probe_eval()
{
	U64 index;

	ETProbes++;

	index = hash % ETSize;

	if (ETable[index].Hash == hash) {
		ETHits++;
		return (ETable[index].Bound);
	}
	return -MATE;
}

void alloc_hash(void) {
	TTable = (TType *) malloc(sizeof(TType) * TTSize);

	if (TTable == NULL) {
		exit(1);
	}

	ETable = (EType *) malloc(sizeof(EType) * ETSize);

	if (ETable == NULL) {
		exit(1);
	}

	return; 
}

void free_hash(void) {
	free(TTable);
	free(ETable);
	return;
}
