/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Funciones de bsqueda del mejor movimiento durante el ponder
*/

movimiento pMotorPiensa(int max_profundidad, int output)
{
	movimiento	m;
	int	i, alpha, beta;
	int contador1, contador2;
	
	para_ponder = FALSO;

	ply = 0;
	nodos = 0;
	contador1 = contador2 = 0;

	memset(pv, 0, sizeof(pv));
	memset(Historia, 0, sizeof(Historia));
	memset(matekiller, 0, sizeof(matekiller));
	memset(killer1, 0, sizeof(killer1));
	memset(killer2, 0, sizeof(killer2));

	#define WINDOW1	15
	#define WINDOW2	150

	alpha = -MATE;
	beta = MATE;

	for (i = 1; i <= max_profundidad;) {

		sigue_Pv = VERDADERO;
		puntos = pPVS(alpha, beta, i, VERDADERO);
		m = pv[0][0];		// este es el mejor movimiento para la profundidad i
		
		if (para_ponder == VERDADERO) {
			while (ply)
				Deshacer();
			return m;
		}

		if (puntos <= alpha) {
			contador1++;
			if (contador1 == 2) {
				alpha = -MATE;
				continue;
			}
			alpha = puntos - WINDOW2;
			continue;
		}
		if (puntos >= beta) {
			contador2++;
			if (contador2 == 2) {
				beta = MATE;
				continue;
			}
			beta = puntos + WINDOW2;
			continue;
		}

		alpha = puntos - WINDOW1;  // configuramos la ventana para el siguiente ciclo
	    beta = puntos + WINDOW1;
	    i++;
		contador1 = contador2 = 0;

		//if (puntos > 9000 || puntos < -9000) //si hay una posicin de mate paramos
		//	break;
	}
	return m; //devolvemos el mejor movimiento
}

int	pPVS(int alpha, int beta, int depth, int hacer_nulo)
{
#define R	2
	int             i, jaque, viejo_alpaso, contar, nuMoves = 0;
	movimiento		mejor_movimiento, lista[218]; //218 es el mximo nmero de combinaciones posibles para un movimiento
	U64	old_hash;

	int ext;
	int mejor = -MATE;

	// TTables
	int bound, htipo;
	movimiento hmov;

	nodos++;

	if ((nodos & 1023) == 0) {	//comprueba cada un cierto nmero de nodos que tenemos tiempo
		if (bioskey()) {
			para_ponder = VERDADERO;
			return (0);
		}
	}

	pv_longuitud[ply] = ply;

	if (ply >= max_profundidad)
		return Evaluar(alpha, beta);

	jaque = EstaEnJaque(turno);

	if (depth <= 0 && !jaque) {
		return pExtiende_capturas(alpha, beta);
	}

	if (ply && repeticion () > 1)
		return 0;

	//comprobamos las tablas hash	
	bound = probe_hash(&htipo, alpha, beta, depth, &hmov);
	if (ply && htipo != HMISS) {
		switch (htipo) {
			case EXACT:
				Actualiza_pv(hmov);
				return bound;
				break;
			case UPPER:
				if (bound <= alpha) {
					Actualiza_pv(hmov);
					return alpha;
				}
				break;
			case LOWER:
				if (bound >= beta) {
					Actualiza_pv(hmov);
					return beta;
				}
				break;
			case avoid_null:
				hacer_nulo = FALSO;
				break;
			default:
				break;
		}
	}


	//probamos un movimiento nulo;
	if (hacer_nulo && !jaque &&!sigue_Pv) {
	
		if ((turno == BLANCO && piezasblanco >= VALOR_CABALLO) || (turno == NEGRO && piezasnegro >= VALOR_CABALLO)) {

			material = piezasblanco+piezasnegro+peonesblanco+peonesnegro;

			depthnull = depth - 1 - R - (material > MEDIOJUEGO) - (depth>6) - (depth>11);
			
			viejo_alpaso= alpaso;
			alpaso = -1;
			regla50++;
			turno = (BLANCO+NEGRO)-turno;
			ply++;
			old_hash = hash;
			poner_hash();
					
			puntos = -pPVS(-beta, -beta+1, depthnull, FALSO);
				
			hash = old_hash;
			ply--;
			turno = (BLANCO+NEGRO)-turno;
			regla50--;
			alpaso = viejo_alpaso;

			if (depth >= 7) {
				if (puntos >= beta) {
					if ((turno == BLANCO && (piezasblanco <= (VALOR_DAMA + VALOR_CABALLO)))
						|| (turno == NEGRO && (piezasnegro <= (VALOR_DAMA + VALOR_CABALLO)))) {

						depthnull = depth - 6;

						puntos = pPVS_no_completo(alpha, beta, depthnull);
					}
				}
			}

			if (puntos >= beta)
				return beta;
		}
	}

    // Internal Iterative Deepening
	if (ply)
		if (hmov.a == 0 && hmov.de == 0 && !hacer_nulo && depth > 4) {
			puntos = pPVS(alpha, beta, MIN(depth-2, depth/2), VERDADERO);
			if (puntos <= alpha)
				puntos = pPVS(-MATE, beta, MIN(depth-2, depth/2), VERDADERO);
			hmov = pv[ply][ply];
		}

	contar = Generar(turno, lista);	// genera todos los movimientos para la posicin actual

	if (sigue_Pv)
		Ordena_pv(contar, lista);

	// recorremos todo el ciclo de movimientos
    for (i = 0; i < contar; ++i) {
		OrdenaMovimiento(i, contar, lista, hmov);
		if (!Hacer_Movimiento(lista[i]))
			continue;
		nuMoves++;

		//extensiones
		ext = 0;
		if (jaque) ext++;
		/*if (pieza[jugadas[njugadas-1].m.a] == PEON && ((turno == NEGRO && FILA(jugadas[njugadas-1].m.a) == 1)
			|| (turno == BLANCO && FILA(jugadas[njugadas-1].m.a) == 6)))
			ext++;
		if(ext > 1) ext = 1;*/
		material = piezasblanco+piezasnegro+peonesblanco+peonesnegro;
		if (material < FINAL && jugadas[njugadas-1].captura > PEON && jugadas[njugadas-1].captura < REY) {
			if (piezasblanco == 0 && piezasnegro == 0) {
				switch(jugadas[njugadas-1].captura) {
					case DAMA:
					case TORRE: ext = 2;
					default: ext = 1;
				}
			}
		}

		if (nuMoves == 1)
			puntos = -pPVS(-beta, -alpha, depth+ext-1, VERDADERO);
		else {
			if (nuMoves >= 4 && depth >= 2 && !ext && !EstaEnJaque(turno) && !sigue_Pv && lista[i].valor < 9830)
				puntos = -pPVS( -alpha-1, -alpha, depth-2, VERDADERO);
			else
				puntos = alpha+1; // Hack to ensure that full-depth search is done
			if (puntos > alpha) {
				puntos = -pPVS( -alpha-1, -alpha, depth+ext-1, VERDADERO);
				if (puntos > alpha && puntos < beta)
					puntos = -pPVS(-beta, -alpha, depth+ext-1, VERDADERO);
			}
		}

		Deshacer();

		if (para_ponder == VERDADERO)
			return (0);

		if (puntos > mejor) {
			mejor = puntos;
			mejor_movimiento = lista[i];
			
			if (mejor > alpha) {
				Historia[mejor_movimiento.de][mejor_movimiento.a] += depth;

				if (mejor >= beta) {
					if ((pieza[lista[i].a] == VACIO) && lista[i].tipo < PROMOCION_A_DAMA) {
						killer2[ply] = killer1[ply];
						killer1[ply] = lista[i];
					}
					if (abs(mejor) > (MATE - 256))
						matekiller[ply] = lista[i];
					// TTables
					store_hash(mejor, alpha, beta, depth, mejor_movimiento);
					return beta;				
				}
				alpha = mejor;
				Actualiza_pv(mejor_movimiento);
				if (!ply && depth > 1) {
					// Mostrar resultados
					print_pv(depth, mejor);
				}
			}
		}
	}

	if (nuMoves == 0) {
		if (jaque) {
			store_hash(-MATE + ply, alpha, beta, depth, mejor_movimiento);
			return -MATE + ply;	//mate
		}
		else {
			store_hash(0, alpha, beta, depth, mejor_movimiento);
			return 0;			//ahogado
		}
	}
	/* evita o busca tablas con la regla de los 50 movimientos */
	if (regla50 >= 100)
		return 0;

	// TTables
	store_hash(mejor, alpha, beta, depth, mejor_movimiento);
	return mejor;
}

int	pPVS_no_completo(int alpha, int beta, int depth)
{
	int             i, contar;
	movimiento		lista[218]; //218 es el mximo nmero de combinaciones posibles para un movimiento

	int mejor = -MATE;

	nodos++;

	pv_longuitud[ply] = ply;

	if (ply >= max_profundidad)
		return Evaluar(alpha, beta);

	contar = Generar(turno, lista);	// genera todos los movimientos para la posicin actual

	if (sigue_Pv)
		Ordena_pv(contar, lista);

	// recorremos todo el ciclo de movimientos
    for (i = 0; i < contar; ++i) {
		OrdenaMovimiento2(i, contar, lista);
		if (!Hacer_Movimiento(lista[i]))
			continue;

		puntos = -pPVS(-beta, -alpha, depth-1, VERDADERO); //nuestra puntuacin es la del contrario con signo contrario

		Deshacer();

		if (puntos > mejor) {
			mejor = puntos;
			if (mejor > alpha) {
				if (mejor >= beta)
					return beta;				
				alpha = mejor;
				Actualiza_pv(lista[i]);
			}
		}
	}
	return mejor;
}

// extensiones es similar a alphabeta salvo en que solo busca capturas, no lleva profundidad
int pExtiende_capturas(int alpha, int beta)
{
	int i, contar2, delta;
	movimiento lista2[218];

	nodos++;

	pv_longuitud[ply] = ply;

	if (ply >= max_profundidad)
		return Evaluar(alpha, beta);

	puntos = Evaluar(alpha, beta);

	if (puntos >= beta)
        return beta;

	if (puntos > alpha)
        alpha = puntos;

    //delta prunning
	material = piezasblanco+piezasnegro+peonesblanco+peonesnegro;
	if (material > FINAL) {
		Evalua_material();
        delta = alpha - 100 - (turno == BLANCO ? (puntosblanco-puntosnegro) : (puntosnegro-puntosblanco));
    } else {
        delta = 0;
    }

    if (delta > VALOR_DAMA) {
        return puntos;
    } else if (delta > VALOR_TORRE) {
        if (turno == BLANCO) {
            if (!damasnegras)
                return puntos;
        } else {
            if (!damasblancas)
                return puntos;
        }

    } else if (delta > VALOR_ALFIL) {
        if (turno == BLANCO) {
            if (!damasnegras && !torresnegras)
                return puntos;
        } else {
            if (!damasblancas && !torresblancas)
                return puntos;
        }
    } else if (delta > VALOR_PEON) {
        if (turno == BLANCO) {
            if (!damasnegras && !torresnegras && !caballosnegros && !alfilesnegros)
                return puntos;
        } else {
            if (!damasblancas && !torresblancas && !caballosblancos && !alfilesblancos)
                return puntos;
        }
    }

	contar2 = Generar_Capturas(turno, lista2);

	if (sigue_Pv)
		Ordena_pv(contar2, lista2);
    
    for (i = 0; i < contar2; ++i) {
		OrdenaMovimiento2(i, contar2, lista2);
		if (abs(valor_pieza[pieza[lista2[i].a]] <= delta) && lista2[i].tipo < PROMOCION_A_DAMA)
			continue;
		if (!Hacer_Movimiento(lista2[i]))
			continue;

		puntos = -pExtiende_capturas(-beta, -alpha); 
		Deshacer();

		if (puntos >= beta)
			return beta;

		if (puntos > alpha){
			alpha = puntos;
			Actualiza_pv(lista2[i]);
		}
	}
	return alpha;
}
