/*
Motor de ajedrez xboard/winboard DanaSah
Pedro Castro

Funcin xboard(), sustituye a la funcin principal si recibimos el parmetro xboard,
gracias a estos comandos, podemos hacer jugar a DanaSah con interfaces de usuario como
Winboard, Arena, Fritz, Chessmaster, etc.
Para comprender dicho protocolo debes examinar la siguiente pgina:
http://www.tim-mann.org/xboard/engine-intf.html
*/

void xboard() /* protocolo xboard*/
{
	char linea[256], comando[256];	//para leer la informacin del GUI y saber que comando llega
	char args[4][64];
	int post;		//utilizada para imprimir la linea principal o no (no utilizada de momento)
	int entrada;	//recibe el nmero de movimientos para el control de tiempo
	int minutos;	//recibe el nmero de minutos base para el control de tiempo
	int incremento;	//recibe el nmero de segundos de incremento para el control de tiempo
	int contar;		//numero de movimientos posibles en una posicin
	movimiento lista[218], mejormovimiento;	//lista de movimientos posibles y mejormovimiento

	printf("feature done=0\n");

	//bucle esperando recibir un comando del GUI
	for (;;) {
		fflush(stdout);	//necesario en el protocolo winboard
        
        if (turno == motor) { // turno del programa
			mejormovimiento = MotorPiensa(max_profundidad,2); //piensa la jugada
			Hacer_Movimiento(mejormovimiento);	//guarda el movimiento en la lista de jugadas
			Enviar_Movimiento(mejormovimiento); //envia el movimiento al GUI 
		    contar = Generar(turno, lista);	  	//genera los movimientos del contrario para saber si ha recibido mate
			imprime_resultado(contar, lista);	//si es final de partida imprime el resultado
			hacer_ponder = VERDADERO;
			continue;
        }
		else {
    		if (ponder && hacer_ponder) {
				mejormovimiento = pMotorPiensa(max_profundidad,2); //piensa la jugada
       			hacer_ponder = FALSO;
    			continue;
            }
        }
		//almacenamos lo que nos llega y lo guardamos en la variable comando
		if (!fgets(linea, 256, stdin))
			return;
		if (linea[0] == '\n')
			continue;
		sscanf(linea, "%s", comando);
		//comprueba si tenemos protocolo winboard 2 y definimos algunas caractersticas
		if (strcmp(comando,"protover") == 0){
			printf("feature sigint=0 sigterm=0 setboard=1 draw=0 colors=0 myname=\"%s\"\n", nombre);
#if defined USE_EGBB
	{
			LoadEgbbLibrary(egbb_path);
	}
#endif
			printf("feature done=1\n");
			continue;
		}
		//nos indica si las caractersticas definidas son aceptadas o no
		if (strcmp(comando, "accepted") == 0) {
			//printf("feature accepted\n");       
			continue;
		}
		if (strcmp(comando, "rejected") == 0) {
			printf("feature rejected\n");       
			continue;
		}
		//si llega xboard simplemente continuamos
		if (strcmp(comando, "xboard") == 0) {
			continue;
		}
		//comprueba si hay que empezar nueva partida
		if (strcmp(comando, "new") == 0) {
			IniciaTablero();
			hacer_ponder = FALSO;
			cerrar_libro();
			abrir_libro();
			continue;
		}
		if (strcmp(comando, "variant") == 0)
			continue;
		//comando para cerrar el motor
		/* en el comando quit tengo un problema, es el mismo comando que en la funcin
		main, aqu intent liberar la memoria pero me da problemas, no se a que es debido */
		if (strcmp(comando, "quit") == 0)
            // TTables           
			//free_hash();
			return;
		if (strcmp(comando, "random") == 0) 
			continue;
		//cambia el color de las piezas entre los jugadores
		if (strcmp(comando, "go") == 0) {
				motor = turno;
				//si es necesario cambiamos de libro de aperturas
				cerrar_libro();
				abrir_libro();
			continue;
		}
		//el motor deja de jugar
		if (strcmp(comando, "force") == 0) {
			motor = VACIO;
			continue;
		}
		//asigna colores, el comando playother es ms moderno que white o black
		if (strcmp(comando, "playother") == 0) {
			if (turno == BLANCO)
				motor = NEGRO;
			else
				motor = BLANCO;
			continue;
		}
		if (strcmp(comando, "white") == 0) {
			turno = BLANCO;
			motor = NEGRO;
			continue;
		}
		if (strcmp(comando, "black") == 0) {
			turno = NEGRO;
			motor = BLANCO;
			continue;
		}
		//parmetros para configurar el nivel de juego del programa
		//level 40 5 0 --> 40 movimientos 5 minutos incremento 0
		//si el nmero de movimientos es 0 la partida es a finish
		if (strcmp(comando, "level") == 0) {
			sscanf(linea, "level %d %d %d", &entrada, &minutos, &incremento);
			mps = entrada;	// nos interesa saber cuantos movimientos para el control
			inc = incremento * 1000;
			//movimientos que quedarn para llegar a dicho control
			/* el contador njugadas se va incrementando tanto para blancas como negras*/
			if (motor == BLANCO)
				quedan = (mps-njugadas/2)-1;
			else
				quedan = (mps-njugadas/2);
			continue;
		}
		//configura el nivel de juego indicando cuantos segundos por movimiento
		// st 10 --> 10 segundos por jugada
		if (strcmp(comando, "st") == 0) {
			sscanf(linea, "st %d", &max_tiempo);
			/*internamente trabajamos con milisegundos*/
			max_tiempo *= 1000;
			max_profundidad = MAX_PROFUNDIDAD;
			continue;
		}
		//configura el nivel de juego indicando la profundidad a jugar
		//sd 8 --> juega hasta profundidad principal 8
		if (strcmp(comando, "sd") == 0) {
			sscanf(linea, "sd %d", &max_profundidad);
			max_tiempo = 1 << 25;	//es como poner tiempo infinito
			continue;
		}
		//recibimos del GUI el tiempo que nos queda
		if (strcmp(comando, "time") == 0) {
			sscanf(linea, "time %d", &max_tiempo);
			/*pasamos a milisegundos que es como trabajamos internamente*/
			max_tiempo *= 10;
			// partidas a finish, suponemos que quedan 30 movimientos para terminar la partida
			if (mps == 0 && inc == 0) {
				max_tiempo /= 30;
			}
			// partidas con incremento de tiempo
			else if (mps == 0 && inc > 0) {
				if (max_tiempo < 2 * inc)
					max_tiempo = 95 * inc / 100;
				else
					max_tiempo = max_tiempo / 25 + inc;
			}
			// si hay un control de tiempo
			else {
				//quitamos un pequeo tiempo para que no pierda en el ltimo movimiento
				max_tiempo = max_tiempo - max_tiempo/mps;
				//una vez llegado al control de tiempo, lo ponemos de nuevo a cero
				if (quedan == 1) {
					max_tiempo = max_tiempo/entrada;
					mps = mps + entrada;
					quedan = entrada;
				}
				//si estamos en medio de un control
				else {
					/*el tiempo para un movimiento ser el tiempo que queda entre los
					movimientos que quedan para el control*/
					max_tiempo = max_tiempo/quedan;
					//nmero de movimientos que quedarn despus de hacer el movimiento
					if (motor == BLANCO)
						quedan = (mps-njugadas/2)-1;
					else
						quedan = (mps-njugadas/2);
				}
			}
			max_profundidad = MAX_PROFUNDIDAD;
			continue;
		}
		//tiempo del contrario, no lo utilizamos de momento
		if (strcmp(comando, "otim") == 0) {
			continue;
		}
		if (strcmp(comando, "usermove") == 0) {
			continue;
		}
		//comando para mover ahora
		if (strcmp(comando, "?") == 0) {
			/*de momento hago mover ahora con la funcin bioskey()*/
			continue;
		}
		//control para winboard, recibimos un ping y enviamos un pong
		if (strcmp(comando,"ping") == 0){
            int n; 
            sscanf(linea, "ping %d", &n);
            printf("pong %d\n",n);
			continue;
		}
		if (strcmp(comando, "draw") == 0) {
			continue;
		}
		if (strcmp(comando, "result") == 0) {
			continue;
		}
		if (strcmp(comando, "setboard") == 0) {
			strcpy(fenstring, "");
			sscanf(linea, "setboard %s %s %s %s", args[0],args[1],args[2],args[3]);
			strcat(fenstring, args[0]);
			strcat(fenstring, args[1]);
			strcat(fenstring, args[2]);
			strcat(fenstring, args[3]);
			SetBoard(fenstring);
			continue;
		}
		//viejo comando edit de winboard para configurar una posicin
		if (strcmp(comando, "edit") == 0) {
			continue;
		}
		if (strcmp(comando, "hint") == 0) {
			char c;
			movimiento mh;
			mh = pv[0][1];
			if (!(mh.de == 0 && mh.a == 0)) {
				//letra a imprimir si hay una promocin
				switch (mh.tipo) {
					case PROMOCION_A_DAMA:
						c = 'q';
						break;
					case PROMOCION_A_TORRE:
						c = 'r';
						break;
					case PROMOCION_A_ALFIL:
						c = 'b';
						break;
					case PROMOCION_A_CABALLO:
						c = 'n';
						break;
					default:
						c = ' ';
				}
				printf("Hint: %c%d%c%d%c", 
					'a' + COLUMNA(mh.de),
					8 - FILA(mh.de),
					'a' + COLUMNA(mh.a),
					8 - FILA(mh.a),
					c);
			}
			continue;
		}
		if (strcmp(comando, "bk") == 0) {
			continue;
		}
		//deshacemos una jugada
		if (strcmp(comando, "undo") == 0) {
			if (njugadas == 0)
				continue;
			Deshacer();
			continue;
		}
		if (strcmp(comando, "remove") == 0) {
			if (njugadas < 2)
				continue;
			Deshacer();
			Deshacer();
			continue;
		}
		//ponder, pensar con el tiempo del contrario
		if (strcmp(comando, "hard") == 0) {
			ponder = VERDADERO;
			continue;
		}
		//no ponder
		if (strcmp(comando, "easy") == 0) {
			ponder = FALSO;
			continue;
		}
		//imprimimos el pensamiento
		if (strcmp(comando, "post") == 0) {
			post = VERDADERO;
			continue;
		}
		//no imprimir el pensamiento
		if (strcmp(comando, "nopost") == 0) {
			post = FALSO;
			continue;
		}
		//modo de anlisis (no funciona del todo todava)
		if (strcmp(comando, "analyze") == 0) {
			max_tiempo = 1<<25;
			max_profundidad = MAX_PROFUNDIDAD;
			mejormovimiento = MotorPiensa(max_profundidad,2);
			continue;
		}
		if (strcmp(comando, "name") == 0) {
			continue;
		}
		if (strcmp(comando, "rating") == 0) {
			continue;
		}
		if (strcmp(comando, "ics") == 0) {
			continue;
		}
		if (strcmp(comando, "computer") == 0) {
			continue;
		}
		if (strcmp(comando, "pause") == 0) {
			continue;
		}
		if (strcmp(comando, "resume") == 0) {
			continue;
		}
		Comprobar_Movimiento(comando);		//comprueba si llega un movimiento del usuario y es legal
		contar = Generar(turno, lista);		//comprueba si tenemos movimientos para hacer
		imprime_resultado(contar, lista);	//si es final imprime un resultado en pantalla
		if (salir == VERDADERO){			//si es final de partida el motor deja de jugar
			motor = VACIO;
			njugadas=0;
			continue;
		}
	}
}
